﻿/* Copyright (c) 2021, NVIDIA CORPORATION. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  * Neither the name of NVIDIA CORPORATION nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


/* The part of source code is used for interoperability between CUDA and OpenGL */


#include "App.h"

#define WINDOWS_LEAN_AND_MEAN
#define NOMINMAX

#include <Windows.h>
#include <string>
#include <iostream>
#include <fstream>
#include <filesystem>


int main() {
	
	std::cout << "Starting app..." << std::endl;

	App app;
	if ( app.NewWindow( SCREEN_WIDTH, SCREEN_HEIGHT, "Bounding Interval Hierarchy Raytracer" ) ) {
		std::string meshFilename;
		bool fileExists = false;
		do {
			std::cout << "Enter mesh filename:" << std::endl;
			std::cin >> meshFilename;
			fileExists = std::filesystem::exists("resources/" + meshFilename + "/" + meshFilename + ".obj");

			if ( fileExists )
				break;
			else
				std::cout << "File doesn't exist. Try again..." << std::endl;

		} while ( !fileExists );
		app.LoadModels( "resources/" + meshFilename + "/" + meshFilename + ".obj");
		app.Run();
	}

	return 0;
}